@@ -9,6 +9,8 @@ use Encode::Alias;
 use parent qw(Exporter);
 use Biber::Date::Format;
 use Text::CSV;
+use Scalar::Util qw (blessed looks_like_number);
+use Unicode::UCD qw(num);
 
 our @EXPORT = qw{
                   $CONFIG_DEFAULT_BIBER
@@ -37,13 +39,16 @@ our @EXPORT = qw{
                   %UNIQUENAME_CONTEXTS
                   %UNIQUENAME_VALUES
                   %MONTHS
+                  %RSTRINGS
+                  %USEDSTRINGS
+                  %YEARDIVISIONS
               };
 
 # Version of biblatex control file which this release expects. Matched against version
 # passed in control file. Used when checking the .bcf
-our $BCF_VERSION = '3.7';
+our $BCF_VERSION = '3.8';
 # Format version of the .bbl. Used when writing the .bbl
-our $BBL_VERSION = '3.1';
+our $BBL_VERSION = '3.2';
 
 # Global flags needed for sorting
 our $BIBER_SORT_FINAL;
@@ -66,26 +71,143 @@ unless ($locale) {
   }
 }
 
+# ISO8601-2 4.8 year divisions
+our %YEARDIVISIONS = ( 21 => 'spring',
+                       22 => 'summer',
+                       23 => 'autumn',
+                       24 => 'winter',
+                       25 => 'springN',
+                       26 => 'summerN',
+                       27 => 'autumnN',
+                       28 => 'winterN',
+                       29 => 'springS',
+                       30 => 'summerS',
+                       31 => 'autumnS',
+                       32 => 'winterS',
+                       33 => 'Q1',
+                       34 => 'Q2',
+                       35 => 'Q3',
+                       36 => 'Q4',
+                       37 => 'QD1',
+                       38 => 'QD2',
+                       39 => 'QD3',
+                       40 => 'S1',
+                       41 => 'S2' );
+
+# Reverse record of macros so we can reverse these for tool mode output
+our %RSTRINGS = ();
+# Record of macros which are actually used in output in tool mode, so that we don't
+# output unused strings.
+our %USEDSTRINGS = ();
+
 our %MONTHS = ('jan' => '1',
-              'feb' => '2',
-              'mar' => '3',
-              'apr' => '4',
-              'may' => '5',
-              'jun' => '6',
-              'jul' => '7',
-              'aug' => '8',
-              'sep' => '9',
-              'oct' => '10',
-              'nov' => '11',
-              'dec' => '12');
+               'feb' => '2',
+               'mar' => '3',
+               'apr' => '4',
+               'may' => '5',
+               'jun' => '6',
+               'jul' => '7',
+               'aug' => '8',
+               'sep' => '9',
+               'oct' => '10',
+               'nov' => '11',
+               'dec' => '12');
 
 # datafieldsets
 our %DATAFIELD_SETS = ();
 
 # datatypes for data model validation
 our %DM_DATATYPES = (
-                     integer  => qr/\A\d+\z/xms,
-                     datepart => qr/\A\d+\z/xms
+                     integer => sub {
+                       my $v = shift;
+                       return 1 if looks_like_number(num($v =~ s/^-//r));
+                       return 0;
+                     },
+                     name => sub {
+                       my $v = shift;
+                       return 1 if (blessed($v) and $v->isa('Biber::Entry::Names'));
+                       return 0;
+                     },
+                     range => sub {
+                       my $v = shift;
+                       return 1 if ref($v) eq 'ARRAY';
+                       return 0;
+                     },
+                     list => sub {
+                       my $v = shift;
+                       return 1 if ref($v) eq 'ARRAY';
+                       return 0;
+                     },
+                     datepart => sub {
+                       my $v = shift;
+                       my $f = shift;
+                       if ($f =~ /timezone$/) {
+                         # ISO 8601
+                         # <time>Z
+                         # <time>±hh:mm
+                         # <time>±hhmm
+                         # <time>±hh
+                         unless ($v eq 'Z' or
+                                 $v =~ m|^[+-]\d\d(?:\\bibtzminsep\s)?(?:\d\d)?$|) {
+                           return 0;
+                         }
+                       }
+                       elsif ($f =~ /season$/) { # LEGACY
+                         return 0 unless $v =~ m/(?:winter|spring|summer|autumn)/
+                       }
+                       elsif ($f =~ /yeardivision$/) {
+                         return 0 unless grep {$v eq $_} values %YEARDIVISIONS;
+#                         return 0 unless $v =~ m/(?:winter[NS]?|spring[NS]?|summer[NS]?|autumn[NS]?|Q[1234]|QD[123]|S[12])/
+                       }
+                       else {
+                         # num() doesn't like negatives
+                         return 0 unless looks_like_number(num($v =~ s/^-//r));
+                       }
+                       return 1;
+                     },
+                     isbn => sub {
+                       my $v = shift;
+                       my $f = shift;
+                       require Business::ISBN;
+
+                       my ($vol, $dir, undef) = File::Spec->splitpath( $INC{"Business/ISBN.pm"} );
+                       $dir =~ s/\/$//; # splitpath sometimes leaves a trailing '/'
+                       # Just in case it is already set. We also need to fake this in tests or it will
+                       # look for it in the blib dir
+                       unless (exists($ENV{ISBN_RANGE_MESSAGE})) {
+                         $ENV{ISBN_RANGE_MESSAGE} = File::Spec->catpath($vol, "$dir/ISBN/", 'RangeMessage.xml');
+                       }
+
+                       my $isbn = Business::ISBN->new($v);
+                       if (not $isbn) {
+                         return 0;
+                       }
+                       return 1;
+                     },
+                     issn => sub {
+                       my $v = shift;
+                       require Business::ISSN;
+
+                       my $issn = Business::ISSN->new($_);
+                       unless ($issn and $issn->is_valid) {
+                         return 0;
+                       }
+                       return 1;
+                     },
+                     ismn => sub {
+                       my $v = shift;
+                       require Business::ISMN;
+                       my $ismn = Business::ISMN->new($_);
+                       unless ($ismn and $ismn->is_valid) {
+                         return 0;
+                       }
+                       return 1;
+                     },
+                     default => sub {
+                       my $v = shift;
+                       return 0 if ref($v);
+                       return 1;
+                     }
                     );
 
 # Mapping of data source and output types to extensions
@@ -139,6 +261,7 @@ our $CONFIG_DEFAULT_BIBER = {
   nolabel                                     => { option => [ {value => q/[\p{Pc}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{S}\p{C}]+/} ] },
 #  nolabelwidthcount                          => { option =>  }, # default is nothing
   nolog                                       => { content => 0 },
+#  nonamestring                               => { option =>  }, # default is nothing
   noskipduplicates                            => { content => 0 },
   nostdmacros                                 => { content => 0 },
   nosort                                      => { option => [ { name => 'setnames', value => q/\A\p{L}{2}\p{Pd}(?=\S)/ },
@@ -146,6 +269,7 @@ our $CONFIG_DEFAULT_BIBER = {
   onlylog                                     => { content => 0 },
   others_string                               => { content => 'others' },
   output_align                                => { content => 0 },
+  output_all_macrodefs                        => { content => 0 },
   output_annotation_marker                    => { content => '+an' },
   output_named_annotation_marker              => { content => ':' },
   output_encoding                             => { content => 'UTF-8' },
@@ -156,6 +280,7 @@ our $CONFIG_DEFAULT_BIBER = {
   output_legacy_dates                         => { content => 0 },
   output_listsep                              => { content => 'and' },
   output_namesep                              => { content => 'and' },
+  output_no_macrodefs                         => { content => 0 },
   output_resolve_xdata                        => { content => 0 },
   output_resolve_crossrefs                    => { content => 0 },
   output_resolve_sets                         => { content => 0 },
@@ -179,6 +304,7 @@ our $CONFIG_DEFAULT_BIBER = {
   validate_config                             => { content => 0 },
   validate_control                            => { content => 0 },
   validate_datamodel                          => { content => 0 },
+  winunicode                                  => { content => 0 },
   wraplines                                   => { content => 0 },
   xdatamarker                                 => { content => 'xdata' },
   xdatasep                                    => { content => '-' },
@@ -555,7 +681,7 @@ L<https://github.com/plk/biber/issues>.
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
-Copyright 2012-2020 Philip Kime, all rights reserved.
+Copyright 2012-2022 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
